function ZombieCanSee(%obj, %target)
{
	%opos = %obj.getEyePoint();
	%th = %target.getEyePoint();
	%tl = %target.getPosition();
	%typemasks = $Typemasks::InteriorObjectType | $Typemasks::TerrainObjectType;
	%types = $Typemasks::PlayerObjectType | $Typemasks::FXbrickAlwaysObjectType;
	%headRaycast = containerRaycast(%opos, %th, %typemasks | %types, %obj);
	%hit = firstWord(%headRaycast);
	if(isObject(%hit) && (%type = %hit.getType()) & %types)
	{
		if(%hit == %target)
		{
			return true;
		}
		else if(%type & $Typemasks::PlayerObjectType)
		{
			return false;
		}
		else //Has to be a brick
		{
			if(!%hit.isRendering())
			{
				return true;
			}
			if(getWord(getColorIDtable(%hit.colorID), 3) < 1)
			{
				return true;
			}
		}
	}
	%legRaycast = containerRaycast(%opos, %tl, %typemasks, %obj);
	%hit = firstWord(%legRaycast);
	if(isObject(%hit) && (%type = %hit.getType()) & %types)
	{
		if(%hit == %target)
		{
			return true;
		}
		else if(%type & $Typemasks::PlayerObjectType)
		{
			return false;
		}
		else
		{
			if(!%hit.isRendering())
			{
				return true;
			}
			if(getWord(getColorIDtable(%hit.colorID), 3) < 1)
			{
				return true;
			}
		}
	}
	return false;
}

function ZombiePathfind(%obj, %target)
{
	%opos = %obj.getEyePoint();
	%th = %target.getEyePoint();
	%tl = %target.getPosition();
	%typemasks = $Typemasks::InteriorObjectType | $Typemasks::TerrainObjectType;
	%types = $Typemasks::PlayerObjectType | $Typemasks::FXbrickAlwaysObjectType;
	%headRaycast = containerRaycast(%opos, %th, %typemasks | %types, %obj);
	%hit = firstWord(%headRaycast);
	if(isObject(%hit) && (%type = %hit.getType()) & %types)
	{
		if(%hit == %target)
		{
			%headCanSee = true;
		}
		else
		{
			%headCanSee = false;
		}
	}
	%legRaycast = containerRaycast(%opos, %tl, %typemasks | %types, %obj);
	%hit = firstWord(%legRaycast);
	if(isObject(%hit) && (%type = %hit.getType()) & %types)
	{
		if(%hit == %target)
		{
			%legsCanSee = true;
		}
		else
		{
			%legsCanSee = false;
		}
	}
	if(%legsCanSee && !%headCanSee)
	{
		return "Crouch";
	}
	if(%headCanSee && !%legsCanSee)
	{
		return "Jump";
	}
	%oz = getWord(%obj.getPosition(), 2);
	%tz = getWord(%tl, 2);
	%ow = %obj.getWaterCoverage() > 0 ? 1 : 0;
	%tw = %target.getWaterCoverage() > 0 ? 1 : 0;
	if(%oz >= %tz + 0.6)
	{
		if(!%ow && !%tw)
		{
			return "None";
		}
		if(%ow && !%tw)
		{
			return "Jump";
		}
		return "Crouch";
	}
	return "Jump";
}

function player::initZombieSearchLoop(%obj)
{
	%obj.mountImage(BlankImage, 2);
	%obj.mountImage(BlankImage, 3);
	%client = %obj.brickGroup.client;
	%mini = %client.minigame;
	if(%mini.zombiesEnabled)
	{
		if(%mini.owner == %client || %mini.useAllPlayersBricks)
		{
			%this = %obj.getDatablock();
			if(%this.normalSearch)
			{
				%obj.ZombieSearchLoop(%mini);
			}
			else
			{
				%this.ZombieSearchLoop(%obj, %mini);
			}
		}
	}
}

function Player::ZombieSearchLoop(%obj, %mini)
{
	if(isObject(%obj) && %obj.getState() !$= "Dead")
	{
		%schedlen = getRandom(1000, 2000);
		cancel(%obj.ZombieSearchSched);
		%obj.ZombieSearchSched = %obj.schedule(%schedlen, ZombieSearchLoop, %mini);
		%obj.mountImage(BlankImage, 3); //For some reason this keeps getting un-equipped
	}
	else
	{
		return;
	}
	%obj.breakBricks();
	%opos = %obj.getEyePoint();
	%radius = %mini.Zombies_SightRadius;
	%hasTarget = (isObject(%target = %obj.target) && %target.getState() !$= "Dead" && vectorDist(%opos, %tpos) > %radius * 2);
	if(%hasTarget)
	{
		if(zombieCanSee(%obj, %target))
		{
			%obj.lastSeenTarget = getSimTime();
			%obj.lastSeenTargetPos = %target.getPosition();
			switch$(ZombiePathfind(%obj, %target))
			{
				case "Jump": %obj.setImageTrigger(2, 1); %obj.schedule(%schedlen / 2, setImageTrigger, 2, 0);
				case "Crouch": %obj.setImageTrigger(3, 1); %obj.schedule(%schedlen / 2, setImageTrigger, 3, 0);
			}
		}
		else
		{
			%obj.setMoveObject(0);
			if(%obj.lastSeenTarget + 5000 < getSimTime())
			{
				%hasTarget = false;
			}
			%obj.setMoveDestination(%obj.lastSeenTargetPos);
			%noWander = true;
		}
	}
	if(!%hasTarget)
	{
		%obj.lastSeenTarget = 0;
		%obj.lastSeenTargetPos = 0;
		%obj.target = 0;
		%obj.setMoveObject(0);
	}
	InitContainerRadiusSearch(%opos, %mini.Zombies_SightRadius, $Typemasks::PlayerObjectType);
	%typemasks = $TypeMasks::FxBrickAlwaysObjectType | $TypeMasks::InteriorObjectType | $TypeMasks::TerrainObjectType | $TypeMasks::VehicleBlockerObjectType;
	while(%hit = ContainerSearchNext())
	{
		%hmini = isObject(%hit.client) ? %hit.client.minigame : %hit.brickGroup.client.minigame;
		if(!%hit.isZombie && %hit != %obj && %hmini == %mini && %hit.getState() !$= "Dead")
		{
			%hpos = %hit.getEyePoint();
			%raycast = containerRaycast(%opos, %hpos, %typemasks);
			if(zombieCanSee(%obj, %hit))
			{
				%opos = %obj.getPosition();
				%hpos = %hit.getPosition();
				%cvec = vectorNormalize(vectorSub(%opos, %hpos));
				if(getWord(vectorSub(%cvec, %obj.getEyeVector()), 2) > 0 || vectorDist(%opos, %hpos) < (%mini.zombies_SightRadius / 4))
				{
					%obj.target = %hit;
					%obj.setMoveObject(%hit);
					if($AddOnLoaded__Emote_Alarm)
					{
						%obj.emote(AlarmProjectile);
					}
					return;
				}
			}
		}
		if(!%noWander)
		{
			%dest = vectorAdd(%opos, (getRandom() - 0.5) * 50 SPC (getRandom() - 0.5) * 50 SPC 0);
			while(isObject(firstWord(containerRaycast(%opos, %dest, %typemasks, %obj))))
			{
				%dest = vectorAdd(%opos, (getRandom() - 0.5) * 20 SPC (getRandom() - 0.5) * 20 SPC 0);
				if(%i++ > 10) //This is here so if the object is encased in bricks or something the server doesn't crash
				{
					break;
				}
			}
			%obj.setMoveDestination(%dest, false);
		}
	}
}

function ZombieCollision(%zombie, %victim, %pos)
{
	serverPlay3D(%pos, ZombieAttackSound);
	%type = %victim.getType();
	%data = %zombie.getDatablock();
	if(%type & $Typemasks::PlayerObjectType)
	{
		if(!%victim.isZombie)
		{
			%zombie.target = %victim;
			%zombie.setMoveObject(%victim);
		}
		if(!%victim.isZombie || %data.damageOtherZombies)
		{
			%zombie.playThread(2, activate2);
			%victim.lastPushTime = getSimTime();
			%initVec = setWord(vectorNormalize(vectorAdd(%zombie.getEyeVector(), %zombie.getForwardVector())), 2, "0.5");
			%fVec = vectorScale(%initVec, %data.pushStrength);
			%victim.setVelocity(vectorAdd(%victim.getVelocity(), %fVec));
			if(isObject(%client = %zombie.getControllingClient()))
			{
				%zombie.setNodeColor($Lhand[%client.lhand], "0.75 0 0 1");
				%zombie.setNodeColor($Rhand[%client.rhand], "0.75 0 0 1");
				%zombie.schedule(5000, zombify);
				%victim.lastPusher = %client;
				%victim.damage(%zombie, %pos, %data.attackPower, %data.damageTypeClient);
			}
			else
			{
				%zombie.target = %victim;
				%zombie.setMoveObject(%victim);
				%zombie.setNodeColor("Lhand", "0.75 0 0 1");
				%zombie.setNodeColor("Rhand", "0.75 0 0 1");
				%zombie.schedule(5000, setNodeColor, "Lhand", %data.color);
				%zombie.schedule(5000, setNodeColor, "Rhand", %data.color);
				%victim.lastPusher = %zombie;
				%victim.lastPushTime = getSimTime();
				%victim.damage(%zombie, %pos, %data.attackPower, %data.damageType);
			}
		}
	}
	if(%type & $Typemasks::VehicleObjectType)
	{
		%victim.damage(%zombie, %pos, %data.attackPower, %data.damageType);
	}
}

function Player::BreakBricks(%obj)
{
	%mini = isObject(%obj.client) ? %obj.client.minigame : %obj.brickGroup.client.minigame;
	if(!%mini.brickDamage)
	{
		return;
	}
	%start = %obj.getEyePoint();
	%vec = vectorScale(%obj.getEyeVector(), 5 * getWord(%obj.getScale(), 2));
	%end = vectorAdd(%start, %vec);
	%ray = containerRaycast(%start, %end, $Typemasks::FxBrickObjectType);
	if(isObject(%brickA = firstWord(%ray)))
	{
		%obj.playThread(2, activate2);
		%bc = %brickA.getGroup().client;
		%bmini = %bc.minigame;
		if(%bmini == %mini && (%mini.useAllPlayersBricks || %bc == %mini.owner))
		{
			%str = %obj.getDatablock().breakStrength;
			if(%brickA.canExplode(%str, %str * 1.5))
			{
				serverPlay3D(posFromRaycast(%ray), ZombieAttackSound);
				%brickA.fakeKillBrick(%vec, %mini.brickRespawnTime / 1000);
				if(isObject(%obj.client))
				{
					%obj.client.incScore(%mini.Points_BreakBrick);
				}
			}
			InitContainerRadiusSearch(posFromRaycast(%ray), getWord(%obj.getScale(), 2) * 2, $Typemasks::FxBrickObjectType);
			while(%brickN = containerSearchNext())
			{
				%nc = %brickN.getGroup().client;
				%nmini = %nc.minigame;
				if(%nmini == %mini && (%mini.useAllPlayersBricks || %nc == %mini.owner))
				{
					if(%brickN.canExplode(%str, %str * 1.5))
					{
						%brickN.fakeKillBrick(%vec, %mini.brickRespawnTime / 1000);
						if(isObject(%obj.client))
						{
							%obj.client.incScore(%mini.Points_BreakBrick);
						}
					}
				}
			}
		}
	}
}

function Player::AttractZombies(%obj, %radius)
{
	if(isObject(%omini = isObject(%oc = %obj.getControllingClient()) ? %oc.minigame : %obj.brickGroup.client.minigame))
	{
		%opos = %obj.getPosition();
		%omini = isObject(%oc = %obj.getControllingClient()) ? %oc.minigame : %obj.brickGroup.client.minigame;
		InitContainerRadiusSearch(%opos, %radius, $Typemasks::PlayerObjectType);
		while(%hit = containerSearchNext())
		{
			if(isObject(%hit.getControllingClient()))
			{
				continue;
			}
			%hmini = %hit.brickGroup.client.minigame;
		}
	}
}